#include "SerialUSB.h"
/*****************************************************************************
* | File      	:   BLE.c
* | Author      :   
* | Function    :   Hardware underlying interface
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2021-11-30
* | Info        :   
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of theex Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include "BLE.h"
#include "CMD.h"
#include "ws2812.pio.h"

uint8_t Data_RX[50] = {0};

/**
 * @brief Query the baud rate of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_Baud_Rate(BluetoothModule *ble) 
{  
    uint8_t i = 0;
    Data_RX[0] = 0;
    uart_puts(UART_ID, Baud_Rate_Query);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'Q') && (Data_RX[1] == 'T'))
    {
        i = (Data_RX[3] - '0') * 10 + Data_RX[4] -'0';
        ble->Baud_Rate = Baud_Rate_ALL[i - 1];
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the low power mode status of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_Power_Low(BluetoothModule *ble) 
{  
    Data_RX[0] = 0;
    uart_puts(UART_ID, Low_Power_Query);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'Q') && (Data_RX[1] == 'L'))
    {
        if(Data_RX[4] == '0')
        {
            ble->Low_Power_Mode = 0;
        }
        else if(Data_RX[4] == '1')
        {
            ble->Low_Power_Mode = 1;
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the name of the BLE connection for the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_Name_BLE(BluetoothModule *ble) 
{ 
    uint8_t i = 3,k=0;
    Data_RX[0] = 0;
    uart_puts(UART_ID, Name_BLE_Query);
    delay(20);
    UART_RX_CMD();
    memset(ble->Name_BLE,0,sizeof(ble->Name_BLE));
    if((Data_RX[0] == 'T') && (Data_RX[1] == 'M'))
    {
        while (Data_RX[i]!='\n')
        {
            i++;
        }
        for(k=3;k<i;k++)
        {
            ble->Name_BLE[k-3] = Data_RX[k];
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the name of the SPP connection for the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_Name_SPP(BluetoothModule *ble) 
{ 
    uint8_t i = 3,k;
    Data_RX[0] = 0;
    uart_puts(UART_ID, Name_SPP_Query);
    delay(20);
    UART_RX_CMD();
    memset(ble->Name_SPP,0,sizeof(ble->Name_SPP));
    if((Data_RX[0] == 'T') && (Data_RX[1] == 'D'))
    {
        while (Data_RX[i]!='\n')
        {
            i++;
        }
        for(k=3;k<i;k++)
        {
            ble->Name_SPP[k-3] = Data_RX[k];
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the Bluetooth module's Bluetooth address (ADD).
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_ADD(BluetoothModule *ble) 
{ 
    uint8_t i = 3,k;
    Data_RX[0] = 0;
    uart_puts(UART_ID, ADD_Query);
    delay(20);
    UART_RX_CMD();
    memset(ble->ADD,0,sizeof(ble->ADD));
    if((Data_RX[0] == 'T') && (Data_RX[1] == 'B'))
    {
        while (Data_RX[i]!='\n')
        {
            i++;
        }
        for(k=3;k<i;k++)
        {
            ble->ADD[k-3] = Data_RX[k];
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the BLE connection switch status of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_BLE_Switch(BluetoothModule *ble) 
{ 
    Data_RX[0] = 0;
    uart_puts(UART_ID, BLE_Switch_Query);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'T') && (Data_RX[1] == '4'))
    {
        if(Data_RX[4] == '1')
        {
            ble->BLE_Switch = 1;
        }
        else if(Data_RX[4] == '0')
        {
            ble->BLE_Switch = 0;
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Query the SPP connection switch status of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
uint8_t Query_SPP_Switch(BluetoothModule *ble) 
{ 
    Data_RX[0] = 0;
    uart_puts(UART_ID, SPP_Switch_Query);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'T') && (Data_RX[1] == '5'))
    {
        if(Data_RX[4] == '1')
        {
            ble->SPP_Switch = 1;
        }
        else if(Data_RX[4] == '0')
        {
            ble->SPP_Switch = 0;
        }
    }
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Set the name of the BLE connection for the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @param name New name for the BLE connection.
 * @return true if the name is successfully set, false on failure.
 */
uint8_t Set_Name_BLE(BluetoothModule *ble,char name[]) 
{ 
    Data_RX[0] = 0;
    char command[32];
    snprintf(command, sizeof(command), Name_BLE_Set, name);
    uart_puts(UART_ID, command);
    delay(20);
    UART_RX_CMD();

    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        memset(ble->Name_BLE,0,sizeof(ble->Name_BLE));
        strcpy(ble->Name_BLE,name);
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Set the name of the SPP connection for the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @param name New name for the SPP connection.
 * @return true if the name is successfully set, false on failure.
 */
uint8_t Set_Name_SPP(BluetoothModule *ble,char name[]) 
{ 
    Data_RX[0] = 0;
    char command[32];
    snprintf(command, sizeof(command), Name_SPP_Set, name);
    uart_puts(UART_ID, command);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        memset(ble->Name_SPP,0,sizeof(ble->Name_SPP));
        strcpy(ble->Name_SPP,name);
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Set the address of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @param add New address to be set for the Bluetooth module.
 * @return true if the address is successfully set, false on failure.
 */
uint8_t Set_ADD(BluetoothModule *ble,char add[]) 
{ 
    Data_RX[0] = 0;
    char command[32];
    snprintf(command, sizeof(command), ADD_SET, add);
    uart_puts(UART_ID, command);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        memset(ble->ADD,0,sizeof(ble->ADD));
        strcpy(ble->ADD,add);
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Set the baud rate of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @param Baud_Rate The desired baud rate to be set.
 * @return true if the baud rate is successfully set, false on failure.
 */
uint8_t Set_Baud_Rate(BluetoothModule *ble,int Baud_Rate) 
{ 
    Data_RX[0] = 0;
    uint8_t dex;
    for (dex = 0; dex < sizeof(Baud_Rate_ALL) / sizeof(Baud_Rate_ALL[0]); dex++)
        if(Baud_Rate == Baud_Rate_ALL[dex])
            break;
    if(dex == 13)
    {
        Serial.print("Invalid baud rate, please reset\r\n");
        return false;
    }
    uart_puts(UART_ID, Baud_Rate_CMD[dex]);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[1] == 'O') && (Data_RX[2] == 'K'))
    {
        ble->Baud_Rate = Baud_Rate;
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true;
}


/**
 * @brief Read and output attributes of the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 */
void Read_Attribute(BluetoothModule *ble)
{
    char buffer[128];
    uint8_t i;
    
    sprintf(buffer, "Bluetooth baud rate is %d", ble->Baud_Rate);
    uart_puts(UART_ID, buffer);
    Serial.printf("Bluetooth baud rate is %d\n", ble->Baud_Rate);
    delay(100);

    if (ble->Low_Power_Mode == 0) {
        uart_puts(UART_ID, "Normal operating mode");
        Serial.printf("Normal operating mode\n");
    }
    else if (ble->Low_Power_Mode == 1) {
        uart_puts(UART_ID, "Low power operation mode");
        Serial.printf("Low power operation mode\n");
    }
    delay(100);

    sprintf(buffer, "BLE name is : %s", ble->Name_BLE);
    uart_puts(UART_ID, buffer);
    Serial.printf("BLE name is : %s\n", ble->Name_BLE);
    delay(100);

    sprintf(buffer, "SPP name is : %s", ble->Name_SPP);
    uart_puts(UART_ID, buffer);
    Serial.printf("SPP name is : %s\n", ble->Name_SPP);
    delay(100);

    sprintf(buffer, "BLE add is : %s", ble->ADD);
    uart_puts(UART_ID, buffer);
    Serial.printf("BLE add is : %s\n", ble->ADD);
    delay(100);

    if (ble->BLE_Switch == 1) {
        uart_puts(UART_ID, "BLE to open up");
        Serial.printf("BLE to open up\n");
    }
    else if (ble->BLE_Switch == 0) {
        uart_puts(UART_ID, "BLE to shut down");
        Serial.printf("BLE to shut down\n");
    }
    delay(100);

    if (ble->SPP_Switch == 1) {
        uart_puts(UART_ID, "SPP to open up");
        Serial.printf("SPP to open up\n");
    }
    else if (ble->SPP_Switch == 0) {
        uart_puts(UART_ID, "SPP to shut down\n");
        Serial.printf("SPP to shut down\n");
    }
    delay(100);
}


/**
 * @brief Enable BLE function.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the BLE function is successfully enabled, false on failure.
 */
uint8_t ON_BLE(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,BLE_ON);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        ble->BLE_Switch = 1;
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief Disable BLE function.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the BLE function is successfully disabled, false on failure.
 */
uint8_t OFF_BLE(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,BLE_OFF);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        ble->BLE_Switch = 0;
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief Enable SPP function.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the SPP function is successfully enabled, false on failure.
 */
uint8_t ON_SPP(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,SPP_ON);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        ble->SPP_Switch = 1;
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief Disable SPP function.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the SPP function is successfully disabled, false on failure.
 */
uint8_t OFF_SPP(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,SPP_OFF);
    delay(20);
    UART_RX_CMD();
    if((Data_RX[0] == 'O') && (Data_RX[1] == 'K'))
    {
        ble->SPP_Switch = 0;
        return true;
    }
    else if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief Enter low power mode for the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the low power mode is successfully enabled, false on failure.
 */
uint8_t ON_Low_Power_Mode(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,Low_Power);
    delay(100);
    UART_RX_CMD();
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    ble->Low_Power_Mode = 1;
    return true; 
}


/**
 * @brief Do not enter low power mode.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the low power mode is successfully disabled, false on failure.
 */
uint8_t OFF_Low_Power_Mode(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID,Not_Low_Power);
    delay(100);
    UART_RX_CMD();
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    ble->Low_Power_Mode = 0;
    return true; 
}


/**
 * @brief Perform a factory reset on the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the factory reset is successful, false on failure.
 */
uint8_t Factory_Reset(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID, Factory_Reset_Cmd);
    delay_ms(200);
    UART_RX_CMD();
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief Reset the Bluetooth module.
 *
 * @param ble Pointer to a BluetoothModule structure.
 * @return true if the reset is successful, false on failure.
 */
uint8_t Reset(BluetoothModule *ble)
{
    Data_RX[0] = 0;
    uart_puts(UART_ID, Reset_Cmd);
    delay_ms(200);
    UART_RX_CMD();
    if((Data_RX[0] == 'E') && (Data_RX[1] == 'R'))
    {
        ERROR_OUT(Data_RX[3]);
        return false;
    }
    return true; 
}


/**
 * @brief GPIO INIT
 * 
 * @param Pin GPIO
 * @param Mode false IN,true OUT
 * @param State dalse DOWN,true UP
 * @return ** void 
 */
void GPIO_Init(uint8_t Pin,uint8_t Mode,uint8_t State)
{
    gpio_init(Pin);
    gpio_set_dir(Pin, Mode);
    if(State == false)
        gpio_pull_down(Pin);
    else
        gpio_pull_up(Pin);
}


/**
 * @brief The serial port is interrupted
 * 
 * @return ** void 
 */
void UART_RX_IRQ()
{
    while (uart_is_readable(UART_ID)) 
    {
        uint8_t ch0 = uart_getc(UART_ID);
        if( uart_is_writable(UART_ID) ) 
        {
            uart_putc(UART_ID, ch0);
            Serial.printf("%c",ch0);
        }
    }
}


/**
 * @brief Instruction error output
 * 
 * @param i Instruction error code
 * @return ** void 
 */
void ERROR_OUT(uint8_t i)
{
    switch (i)
    {
        case '1':Serial.printf("Incorrect data frame received\r\n");break;
        case '2':Serial.printf("The received command does not exist\r\n");break;
        case '3':Serial.printf("Received AT instruction, carriage return line feed not received\r\n");break;
        case '4':Serial.printf("Sent instructions with parameters that are out of range or in the wrong format. Please check your AT instructions\r\n");break;
        case '7':Serial.printf("The MCU sends data to the mobile phone, but notify is not enabled on the mobile phone. The BLE connection is successfu\r\n");break;
        default:
            break;
    }
}


/**
 * @brief Serial Port Initialization
 * 
 * @param Uart_Irq Enable or disable the serial port
 * @return ** void 
 */
void UART_Init(bool Uart_Irq)
{
    uart_init(UART_ID, BAUD_RATE);
    gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
    gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);
    uart_set_hw_flow(UART_ID, false, false);
    uart_set_format(UART_ID, DATA_BITS, STOP_BITS, PARITY);
    uart_set_fifo_enabled(UART_ID,true);
    if(Uart_Irq == true)
    {
        int UART_IRQ = UART_ID == uart0 ? UART0_IRQ : UART1_IRQ;
        irq_set_exclusive_handler(UART_IRQ, UART_RX_IRQ);
        irq_set_enabled(UART_IRQ, true);
        uart_set_irq_enables(UART_ID, true, false);
    }
}


/**
 * @brief Bluetooth initialization
 * 
 * @return ** void 
 */
void BLE_Init(BluetoothModule *ble)
{
    // PIO init
    PIO pio = pio0;
    int sm = 0;
    uint offset = pio_add_program(pio, &ws2812_program);
    ws2812_program_init(pio, sm, offset, 16, 800000, true);

    GPIO_Init(BLE_MODE_PIN,GPIO_IN,GPIO_DOWN);

    while(gpio_get(BLE_MODE_PIN) == 0)
    {
        Serial.printf("Please connect the device with your mobile phone\r\n");
        delay(1000);
    }

    // After power-on, the receiver will generate an interference. Remove the interference first
    uart_puts(UART_ID, "Remove the interference");
    delay(20);
    UART_RX_CMD();

    // Check whether notify is enabled on the mobile phone
    while(1)
    {
        uart_puts(UART_ID, "Connect the test");
        delay(20);
        UART_RX_CMD();
        if((Data_RX[0] == 'E') && (Data_RX[1] == 'R') && (Data_RX[2] == '+') && (Data_RX[3] == '7'))
            Serial.printf("Enable notify on the mobile phone\r\n");
        else
        {
            Serial.printf("The connection is successful\r\n");
            uart_puts(UART_ID, "The connection is successful");
            delay(100);
            break;
        }
        Data_RX[3] = 0;
        delay(1000);
    }

    // Querying Basic Information
    Query_Baud_Rate(ble);
    Query_Power_Low(ble);
    Query_Name_BLE(ble);
    Query_Name_SPP(ble);
    Query_ADD(ble);
    Query_BLE_Switch(ble);
    Query_SPP_Switch(ble);
    Read_Attribute(ble);

    // Change the name of Bluetooth
    char newName_BLE[] = "BLE-Waveshare";
    if( true == Set_Name_BLE(ble,newName_BLE) )
    {
        char buffer[128];
        sprintf(buffer, "BLE was successfully renamed to : %s", ble->Name_BLE);
        uart_puts(UART_ID,buffer);
        Serial.printf("BLE was successfully renamed to : %s\n", ble->Name_BLE);
    }
    delay(100);
    char newName_SPP[] = "SPP-Waveshare";
    if( true == Set_Name_SPP(ble,newName_SPP) )
    {
        char buffer[128];
        sprintf(buffer, "SPP was successfully renamed to : %s", ble->Name_SPP);
        uart_puts(UART_ID,buffer);
        Serial.printf("SPP was successfully renamed to : %s\n", ble->Name_SPP);
    }
    delay(100);
}


/**
 * @brief Instruction special serial port receiving and processing
 * 
 * @return ** void 
 */
void UART_RX_CMD()
{
    uint8_t i = 0;
    while(uart_is_readable(UART_ID))
    {
        uint8_t ch = uart_getc(UART_ID);
        sleep_ms(1);
        if((uart_is_readable(UART_ID) == 0) || (ch == '\0'))
        {
            Data_RX[i++] = '\n';
        }
        Data_RX[i++] = ch;
    }
}


/**
 * @brief Serial port reception processing
 * 
 * @return ** void 
 */
void UART_Read_Write()
{
    uint8_t rx_data[5]={0},rx_num=0;
    while(uart_is_readable(UART_ID))
    {
        uint8_t ch = uart_getc(UART_ID);
        uart_putc(UART_ID,ch);
        Serial.printf("%c",ch);
        sleep_ms(1);
        if(rx_num < 5 )
          rx_data[rx_num]=ch;
        rx_num++;
        if(uart_is_readable(UART_ID) == 0)
        {
            if( true == My_Cmd(rx_num,rx_data))
            {
                uart_puts(UART_ID," cmd takes effect!");
            }
            Serial.printf("\r\n");
        }
    }
}


/**
 * @brief Serial port transmission processing
 * 
 * @param ch character string
 * @param cmd Command or not
 * @return ** void 
 */
void UART_TX(uint8_t *ch)
{
    GPIO_Init(BLE_MODE_PIN,GPIO_IN,GPIO_DOWN);
    if(gpio_get(BLE_MODE_PIN) == 0)
        Serial.printf("Please connect the device with your mobile phone");
    else
        uart_puts(UART_ID, (const char*)ch);
}


void put_pixel(uint32_t pixel_grb)
{
    pio_sm_put_blocking(pio0, 0, pixel_grb << 8u);
}


void put_rgb(uint8_t red, uint8_t green, uint8_t blue)
{
    uint32_t mask = (green << 16) | (red << 8) | (blue << 0);
    put_pixel(mask);
}


void RGB_Red()
{
    put_rgb(0,0xff,0);
}


void RGB_Green()
{
    put_rgb(0xff,0,0);
}


void RGB_Blue()
{
    put_rgb(0,0,0xff);
}


void RGB_Close()
{
    put_rgb(0,0,0);
}


uint8_t My_Cmd(uint8_t rx_num,uint8_t rx_data[])
{
    if(rx_num == 3 && rx_data[0] == 'u' && rx_data[1] == 's' && rx_data[2] == 'b')
    {
        set_sys_clock_khz(50000, true);
        return true;
    }
    else if(rx_num == 3 && rx_data[0] == 'r' && rx_data[1] == 'e' && rx_data[2] == 'd')
    {
        RGB_Red();
        return true;
    }
    else if(rx_num == 4 && rx_data[0] == 'b' && rx_data[1] == 'l' && rx_data[2] == 'u' && rx_data[3] == 'e')
    {
        RGB_Blue();
        return true;
    }
    else if(rx_num == 5 && rx_data[0] == 'c' && rx_data[1] == 'l' && rx_data[2] == 'o' && rx_data[3] == 's' && rx_data[4] == 'e')
    {
        RGB_Close();
        return true;
    }
    else if(rx_num == 5 && rx_data[0] == 'g' && rx_data[1] == 'r' && rx_data[2] == 'e' && rx_data[3] == 'e' && rx_data[4] == 'n')
    {
        RGB_Green();
        return true;
    }
    return false;
}
