/*****************************************************************************
* | File      	:   BLE_test.c
* | Author      :   
* | Function    :   test Demo
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2021-11-30
* | Info        :   
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of theex Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include "BLE_Test.h"
#include "BLE.h"

#include "pico/sleep.h"
#include "hardware/rtc.h"

#define VREG_VSEL VREG_VOLTAGE_0_90

uint8_t sleep_time = 5;

static void rtc_sleep(void) {
    // Start on Friday 5th of June 2020 15:45:00
    datetime_t t = {
            .year  = 2020,
            .month = 06,
            .day   = 05,
            .dotw  = 5, // 0 is Sunday, so 5 is Friday
            .hour  = 15,
            .min   = 45,
            .sec   = 00
    };

    // Start the RTC
    rtc_init();
    rtc_set_datetime(&t);
    t.sec += sleep_time;
    sleep_goto_sleep_until(&t, NULL);
}

void BLE_test()
{
    stdio_init_all();
    vreg_set_voltage(VREG_VSEL);
    sleep_ms(10);
    set_sys_clock_khz(20000, true);
    BluetoothModule *ble = (BluetoothModule *)malloc(sizeof(BluetoothModule));
    UART_Init(UART_IRQ_OFF);
    BLE_Init(ble);

    while (1) {
        uint64_t start_time = time_us_64();
        uint64_t duration = 5 * 1000000;

        while (time_us_64() - start_time < duration) {
            UART_Read_Write();
        }

        rtc_sleep();
    }
}



