/*****************************************************************************
* | File      	:   BLE.h
* | Author      :   
* | Function    :   Hardware underlying interface
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2021-11-30
* | Info        :   
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of theex Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#ifndef _BLE_H_
#define _BLE_H_

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/uart.h"
#include "hardware/irq.h"

#include "hardware/pio.h"
#include "hardware/clocks.h"
#include "hardware/vreg.h"


typedef struct {  
    int Baud_Rate;  
    uint8_t Low_Power_Mode;   
    uint8_t Name_BLE[64];  
    uint8_t Name_SPP[64];  
    uint8_t ADD[64];  
    uint8_t BLE_Switch;  
    uint8_t SPP_Switch;  
} BluetoothModule; 

/**
 * @brief A serial port ID
 */
#define UART_ID uart1

/**
 * @brief A serial port configuration
 */
#define BAUD_RATE 115200
#define DATA_BITS 8
#define STOP_BITS 1
#define PARITY    UART_PARITY_NONE

/**
 * @brief A serial port pins
 */
#define UART_TX_PIN 20
#define UART_RX_PIN 21

/**
 * @brief Bluetooth connection detection pin
 */
#define BLE_MODE_PIN 15

#define GPIO_DOWN       false
#define GPIO_UP         true
#define UART_IRQ_OFF    false
#define UART_IRQ_ON     true

void delay(uint32_t s);
void delay_ms(uint32_t ms);
uint8_t Query_Baud_Rate(BluetoothModule *ble);
uint8_t Query_Power_Low(BluetoothModule *ble);
uint8_t Query_Name_BLE(BluetoothModule *ble);
uint8_t Query_Name_SPP(BluetoothModule *ble);
uint8_t Query_ADD(BluetoothModule *ble);
uint8_t Query_BLE_Switch(BluetoothModule *ble);
uint8_t Query_SPP_Switch(BluetoothModule *ble);
uint8_t Set_Baud_Rate(BluetoothModule *ble,int Baudrate);
uint8_t Set_Name_BLE(BluetoothModule *ble,char name[]);
uint8_t Set_Name_SPP(BluetoothModule *ble,char name[]);
uint8_t Set_ADD(BluetoothModule *ble,char add[]);
void Read_Attribute(BluetoothModule *ble);
void BLE_Init(BluetoothModule *ble);
void GPIO_Init(uint8_t Pin,uint8_t Mode,uint8_t State);
void UART_RX_IRQ(void);
void UART_Init(bool Uart_Irq);
void UART_RX_CMD(void);
void UART_Read_Write(void);
void UART_TX(uint8_t *ch);
void ERROR_OUT(uint8_t i);
uint8_t ON_BLE(BluetoothModule *ble);
uint8_t OFF_BLE(BluetoothModule *ble);
uint8_t ON_SPP(BluetoothModule *ble);
uint8_t OFF_SPP(BluetoothModule *ble);
uint8_t ON_Low_Power_Mode(BluetoothModule *ble);
uint8_t OFF_Low_Power_Mode(BluetoothModule *ble);
uint8_t Reset(BluetoothModule *ble);
uint8_t Factory_Reset(BluetoothModule *ble);
void put_pixel(uint32_t pixel_grb);
void put_rgb(uint8_t red, uint8_t green, uint8_t blue);
void RGB_Red(void);
void RGB_Green(void);
void RGB_Blue(void);
void RGB_Close(void);
uint8_t My_Cmd(uint8_t rx_num,uint8_t rx_data[]);


#endif
