from WS2812 import WS2812
import time

class BluetoothModule:
    
    def __init__(self, uart):
        self.uart = uart
        self.Baudrate = 0
        self.Low_Power = 0
        self.Name_BLE = ""
        self.Name_SPP = ""
        self.ADD = ""
        self.BLE_Switch = 0
        self.SPP_Switch = 0
        self.rgb = WS2812()
        
    def My_Cmd(self,rxData):
        rxData = rxData.decode('utf-8')
        if rxData in ["red", "green", "blue", "close"]:
            if rxData == "red":
                self.rgb.RGB_Red()
            elif rxData == "green":
                self.rgb.RGB_Green()
            elif rxData == "blue":
                self.rgb.RGB_Blue()
            elif rxData == "close":
                self.rgb.RGB_Close()
            return True
        return False
        
    def Write(self, string):
        self.uart.write(string)
        
    def Read(self, length=None):
        if length is None:
            Data_RX = self.uart.read()
        else:
            Data_RX = self.uart.read(length)
        return Data_RX
    
    def Read_Write(self):
        Data_RX = self.uart.read()
        if Data_RX is not None:
            self.uart.write(Data_RX)
        return Data_RX
    
    def Query_Baud_Rate(self):
        self.uart.write(b"AT+QT\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if Data_RX[0:3] == b'QT+':
            i = (Data_RX[3] - 48) * 10 + Data_RX[4] - 48
            baud_rates = [
                "9600", "19200", "38400", "57600", "115200",
                "256000", "512000", "230400", "460800", "1000000",
                "31250", "2400", "4800"
            ]
            if 1 <= i <= 13:
                self.Baudrate = baud_rates[i - 1]
                    
    def Query_Low_Power(self):
        self.uart.write(b"AT+QL\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:5] == b'QL+00' ):
            self.Low_Power = 0
        elif( Data_RX[0:5] == b'QL+01' ):
            self.Low_Power = 1
            
    def Query_Name_BLE(self):
        self.uart.write(b"AT+TM\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:3] == b'TM+' ):
            i = 3
            while (chr(Data_RX[i]) != '\r'):
                i = i+1
            self.Name_BLE = Data_RX[3:i]
        
    def Query_Name_SPP(self):
        self.uart.write(b"AT+TD\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:3] == b'TD+' ):
            i = 3
            while (chr(Data_RX[i]) != '\r'):
                i = i+1
            self.Name_SPP = Data_RX[3:i]
        
    def Query_ADD(self):
        self.uart.write(b"AT+TN\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:3] == b'TB+' ):
            i = 3
            while (chr(Data_RX[i]) != '\r'):
                i = i+1
            self.ADD = Data_RX[3:i]
        
    def Query_BLE_Switch(self):
        self.uart.write(b"AT+T4\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:5] == b'T4+01' ):
            self.BLE_Switch = 1
        elif( Data_RX[0:5] == b'T4+00' ):
            self.BLE_Switch = 0
        
    def Query_SPP_Switch(self):
        self.uart.write(b"AT+T5\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if Data_RX is None:
            return False
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        if( Data_RX[0:5] == b'T5+01' ):
            self.SPP_Switch = 1
        elif( Data_RX[0:5] == b'T5+00' ):
            self.SPP_Switch = 0
            
    def Set_Name_BLE(self, name=None):
        if name is None:
            command = b"AT+BMBLE-Waveshare\r\n"
        else:
            command = b"AT+BM%s\r\n" % name.encode('utf-8')
            
        self.uart.write(command)
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def Set_Name_SPP(self, name=None):
        if name is None:
            command = b"AT+BDSPP-Waveshare\r\n"
        else:
            command = b"AT+BD%s\r\n" % name.encode('utf-8')
           
        self.uart.write(command)
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def Set_ADD(self,address=None):
        if address is None:
            command = b"AT+BN112233445566\r\n"
        else:
            command = b"AT+BN%s\r\n" % address.encode('utf-8')#12?
            
        self.uart.write(command)
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def Set_Baudrate(self,Baudrate):
        if(Baudrate == 9600):
            self.uart.write(b"AT+CT01\r\n")
        elif(Baudrate == 19200):
            self.uart.write(b"AT+CT02\r\n")
        elif(Baudrate == 38400):
            self.uart.write(b"AT+CT03\r\n")
        elif(Baudrate == 57600):
            self.uart.write(b"AT+CT04\r\n")
        elif(Baudrate == 115200):
            self.uart.write(b"AT+CT05\r\n")
        elif(Baudrate == 256000):
            self.uart.write(b"AT+CT06\r\n")
        elif(Baudrate == 512000):
            self.uart.write(b"AT+CT07\r\n")
        elif(Baudrate == 230400):
            self.uart.write(b"AT+CT08\r\n")
        elif(Baudrate == 460800):
            self.uart.write(b"AT+CT09\r\n")
        elif(Baudrate == 1000000):
            self.uart.write(b"AT+CT10\r\n")
        elif(Baudrate == 31250):
            self.uart.write(b"AT+CT11\r\n")
        elif(Baudrate == 2400):
            self.uart.write(b"AT+CT12\r\n")
        elif(Baudrate == 4800):
            self.uart.write(b"AT+CT13\r\n")       
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def Read_Attributes(self):
        # Baudrate
        print("Bluetooth baud rate is " + self.Baudrate)
        self.uart.write("Bluetooth baud rate is " + self.Baudrate)
        time.sleep_ms(100)
        # Mode
        if( self.Low_Power == 0 ):
            print("Normal operating mode")
            self.uart.write("Normal operating mode")
        elif( self.Low_Power == 1 ):
            print("Low power operation mode")
            self.uart.write("Low power operation mode")
        time.sleep_ms(100)
        # BLE name
        print("BLE name is : ")
        self.uart.write("BLE name is : ")
        print("%s"%self.Name_BLE)
        self.uart.write(self.Name_BLE)
        time.sleep_ms(100)
        # SPP name
        print("SPP name is : ")
        self.uart.write("SPP name is : ")
        print("%s"%self.Name_SPP)
        self.uart.write(self.Name_SPP)
        time.sleep_ms(100)
        # BLE ADD
        print("BLE add is : ")
        self.uart.write("BLE add is : ")
        print("%s"%self.ADD)
        self.uart.write(self.ADD)
        time.sleep_ms(100)
        # BLE Switch
        if( self.BLE_Switch == 1 ):
            print("BLE to open up")
            self.uart.write("BLE to open up")
        elif( self.BLE_Switch == 0 ):
            print("BLE to shut down")
            self.uart.write("BLE to shut down")
        time.sleep_ms(100)
        # SPP Switch  
        if( self.SPP_Switch == 1 ):
            print("SPP to open up");
            self.uart.write("SPP to open up")
        elif( self.SPP_Switch == 0 ):
            print("SPP to shut down")
            self.uart.write("SPP to shut down")
        time.sleep_ms(100)
        
    def ON_BLE(self):
        self.uart.write(b"AT+B401\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def OFF_BLE(self):
        self.uart.write(b"AT+B400\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def ON_SPP(self):
        self.uart.write(b"AT+B501\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def OFF_SPP(self):
        self.uart.write(b"AT+B500\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        elif( Data_RX[0:2] == b'OK' ): 
#           print("Command executed successfully")
            return True
        
    def ON_Low_Power_Mode(self):
        self.uart.write(b"AT+CL01\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        
    def OFF_Low_Power_Mode(self):
        self.uart.write(b"AT+CL00\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        
    def Reset(self):
        self.uart.write(b"AT+CZ\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False
        
    def Factory_Reset(self):
        self.uart.write(b"AT+CW\r\n")
        time.sleep_ms(20)
        Data_RX = self.uart.read()
        if( Data_RX[0:3] == b'ER+' ):
            self.ERROR_OUT( Data_RX[3] - 48 )
            return False

    def ERROR_OUT(self, data):
        if data == 1:
            print("Incorrect data frame received")
        elif data == 2:
            print("The received command does not exist")
        elif data == 3:
            print("Received AT instruction, carriage return line feed not received")
        elif data == 4:
            print("Sent instructions with parameters that are out of range or in the wrong format. Please check your AT instructions")
        elif data == 7:
            print("The MCU sends data to the mobile phone, but notify is not enabled on the mobile phone. The BLE connection is successfu")

